==> openGion8.2.0.3_src.zip: uap/webapps/gf/src/org/opengion/plugin/io/TableReader_Excel.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.io;

import org.opengion.fukurou.system.OgRuntimeException ;		// 6.4.2.0 (2016/01/29)
import java.io.File;										// 6.2.0.0 (2015/02/27)
import java.util.Arrays;									// 6.8.2.4 (2017/11/20)

import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.FileInfo;					// 6.2.4.2 (2015/05/29)
import org.opengion.fukurou.model.TableModelHelper;			// 6.2.0.0 (2015/02/27) クラス名変更
import org.opengion.fukurou.model.EventReader_XLS;			// 6.2.4.2 (2015/05/29)
import org.opengion.fukurou.model.EventReader_XLSX;			// 6.2.4.2 (2015/05/29)
// import org.opengion.fukurou.model.FileOperation;			// 8.0.0.1 (2021/10/08)
// import org.opengion.fukurou.util.FileUtil;					// 8.0.0.1 (2021/10/08)

import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.io.AbstractTableReader;		// 6.2.0.0 (2015/02/27)
// import org.opengion.hayabusa.io.HybsFileOperationFactory;	// 8.0.0.1 (2021/10/08)

import static org.opengion.fukurou.system.HybsConst.CR ;	// 6.2.2.0 (2015/03/27)

/**
 * POI による、EXCELバイナリファイルを読み取る実装クラスです。
 *
 * ファイル名、シート名を指定して、データを読み取ることが可能です。
 * 第一カラムが # で始まる行は、コメント行なので、読み飛ばします。
 * カラム名の指定行で、カラム名が null の場合は、その列は読み飛ばします。
 *
 * 入力形式は、openXML形式にも対応しています。
 * ファイルの内容に応じて、.xlsと.xlsxのどちらで読み取るかは、内部的に
 * 自動判定されます。
 *
 * @og.rev 3.5.4.8 (2004/02/23) 新規作成
 * @og.rev 4.3.6.7 (2009/05/22) ooxml形式対応
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class TableReader_Excel extends AbstractTableReader {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "8.0.0.2 (2021/10/15)" ;

	/**
	 * デフォルトコンストラクター
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public TableReader_Excel() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * DBTableModel から 各形式のデータを作成して,BufferedReader より読み取ります。
	 * コメント/空行を除き、最初の行は、必ず項目名が必要です。
	 * それ以降は、コメント/空行を除き、データとして読み込んでいきます。
	 * このメソッドは、EXCEL 読み込み時に使用します。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @og.rev 5.1.6.0 (2010/05/01) columns 処理 追加
	 * @og.rev 5.1.6.0 (2010/05/01) skipRowCountの追加
	 * @og.rev 5.1.8.0 (2010/07/01) Exception をきちっと記述(InvalidFormatException)
	 * @og.rev 5.2.1.0 (2010/10/01) setTableColumnValues メソッドを経由して、テーブルにデータをセットする。
	 * @og.rev 5.5.1.2 (2012/04/06) HeaderData を try の上にだし、エラーメッセージを取得できるようにする。
	 * @og.rev 5.5.7.2 (2012/10/09) sheetNos 追加による複数シートのマージ読み取りサポート
	 * @og.rev 5.5.8.2 (2012/11/09) HeaderData に デバッグフラグを渡します。
	 * @og.rev 6.0.2.5 (2014/10/31) debug=true 時に、進捗が見えるようにします。
	 * @og.rev 6.1.0.0 (2014/12/26) Excel関係改善
	 * @og.rev 6.2.0.0 (2015/02/27) TableReader クラスの呼び出し元メソッドの共通化(EXCEL,TEXT)。新規
	 * @og.rev 6.2.1.0 (2015/03/13) TableReaderModel を外部からセットします。
	 * @og.rev 6.2.3.0 (2015/05/01) 行読み飛ばし nullSkipClm追加
	 * @og.rev 6.2.4.2 (2015/05/29) POIUtil を使わず、EventReader_XLS、EventReader_XLSX を直接呼び出します。
	 * @og.rev 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
	 * @og.rev 8.0.0.1 (2021/10/08) OPCPackage が、ｸﾗｳﾄﾞﾌｧｲﾙを処理できない…みたい。
	 * @og.rev 8.0.0.2 (2021/10/15) TableReader系は、ｸﾗｳﾄﾞから、ﾛｰｶﾙﾌｧｲﾙに移してから処理する。
	 *
	 * @param   file 読み取り元ファイル名(ﾛｰｶﾙﾌｧｲﾙ)
	 * @param   enc ファイルのエンコード文字列(未使用)
	 */
	@Override
	public void readDBTable( final File file , final String enc ) {
		final boolean isDebug = isDebug();			// 5.5.7.2 (2012/10/09) デバッグ情報

	//	if( isDebug ) { System.out.println( " Filename=" + file ) ; }

//		// 8.0.0.1 (2021/10/08) OPCPackage が、ｸﾗｳﾄﾞﾌｧｲﾙを処理できない…みたい。
//		// ｸﾗｳﾄﾞのﾌｧｲﾙをﾛｰｶﾙにｺﾋﾟｰしています。
//		final File file = new File(inFile.toString());		// 後で修正
//		if( inFile instanceof FileOperation ) {
//			FileUtil.copy( inFile,file );
//		}

		final TableModelHelper helper = new TableModelHelper() {
			private boolean[] useShtNo ;			// 6.1.0.0 (2014/12/26) 読み取り対象のシート管理

			/**
			 * シートの数のイベントが発生します。
			 *
			 * 処理の開始前に、シートの数のイベントが発生します。
			 * これを元に、処理するシート番号の選別が可能です。
			 * 初期実装は、されていません。
			 *
			 * @og.rev 6.1.0.0 (2014/12/26) Excel関係改善
			 * @og.rev 6.2.6.0 (2015/06/19) #csv2ArrayExt(String,int)の戻り値を、文字列配列から数字配列に変更。
			 * @og.rev 5.9.9.2 (2016/06/17) シート名指定が効果ないため修正
			 *
			 * @param   size  シートの数
			 */
			@Override
			public void sheetSize( final int size ) {
				if( isDebug ) { System.out.println( " sheetSize=" + size ) ; }
				// 5.5.7.2 (2012/10/09) 複数シートのマージ読み取り。 sheetNos の指定が優先される。
				useShtNo = new boolean[size];			// シート数だけ、配列を作成する。
				if( sheetNos != null && sheetNos.length() > 0 ) {
					Integer[] sheetList = StringUtil.csv2ArrayExt( sheetNos , size-1 );	// 最大シート番号は、シート数-1
					for( int i=0; i<sheetList.length; i++ ) {
						useShtNo[sheetList[i]] = true;	// 読み取り対象のシート番号のみ、ture にセット
					}
				}
				else if ( sheetName == null || sheetName.length() == 0 ) { //  5.9.9.2 (2016/06/17) sheetNameを見ないと必ず先頭が読み込まれる
					useShtNo[0] = true;					// 一番目のシート
				}
			}

			/**
			 * シートの読み取り開始時にイベントが発生します。
			 *
			 * 新しいシートの読み取り開始毎に、１回呼ばれます。
			 * 戻り値が、true の場合は、そのシートの読み取りを継続します。
			 * false の場合は、そのシートの読み取りは行わず、次のシートまで
			 * イベントは発行されません。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 7.3.1.1 (2021/02/25) 現在実行中のシート名をセットする
			 *
			 * @param   shtNm  シート名
			 * @param   shtNo  シート番号(0～)
			 * @return  true:シートの読み取り処理を継続します/false:このシートは読み取りません。
			 */
			@Override
			public boolean startSheet( final String shtNm,final int shtNo ) {
	//			if( isDebug ) { System.out.println( " Sheet[" + shtNo + "]=" + shtNm ) ; }
				super.startSheet( shtNm , shtNo );		// cnstData の呼び出しの為。無しで動くようにしなければ…

//				return  useShtNo  != null && useShtNo[shtNo] ||
//						sheetName != null && sheetName.equalsIgnoreCase( shtNm ) ;				// 6.9.7.0 (2018/05/14) PMD Useless parentheses.

				final boolean isUseSheet = useShtNo  != null && useShtNo[shtNo] ||
									sheetName != null && sheetName.equalsIgnoreCase( shtNm ) ;	// 6.9.7.0 (2018/05/14) PMD Useless parentheses.

				if( isUseSheet ) { shtName( shtNm ); }	// 7.3.1.1 (2021/02/25)

				return isUseSheet;
			}

			/**
			 * カラム名配列がそろった段階で、イベントが発生します。
			 *
			 * openGion での標準的な処理は、colNo==0 の時に、val の先頭が、#NAME
			 * で始まるレコードを、名前配列として認識します。
			 * #value( String,int,int ) で、この #NAME だけは、継続処理されます。
			 * その上で、#NAME レコードが終了した時点で、カラム名配列が完成するので
			 * そこで初めて、このメソッドが呼ばれます。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 6.1.0.0 (2014/12/26) omitNames 属性を追加
			 * @og.rev 6.8.2.4 (2017/11/20) isDebug 処理を追加
			 *
			 * @param   names  カラム名配列(可変長引数)
			 * @see		#value( String,int,int )
			 */
			@Override
			public void columnNames( final String[] names ) {
				if( isDebug ) { System.out.println( " names=" + Arrays.toString( names ) ) ; }
				setTableDBColumn( names ) ;
			}

			/**
			 * #NAME のオリジナルカラム名配列がそろった段階で、イベントが発生します。
			 *
			 * @og.rev 7.3.1.3 (2021/03/09) #NAMEのオリジナルを取得できるようにします。
			 *
			 * @param   names  カラム名配列
			 */
			@Override
			public void originalNames( final String[] names ) {
				setOriginalNames( names ) ;
			}

			/**
			 * row にあるセルのオブジェクト値がそろった段階で、イベントが発生します。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 6.2.1.0 (2015/03/13) setTableColumnValuesに、行番号を引数に追加
			 * @og.rev 6.8.2.4 (2017/11/20) isDebug 処理を追加
			 *
			 * @param   vals    文字列値の１行分の配列
			 * @param   rowNo   行番号(0～)
			 */
			@Override
			public void values( final String[] vals,final int rowNo ) {
				if( isDebug && rowNo % 100 == 0 ) { System.out.println( "   rowNo=" + rowNo ) ; }
				setTableColumnValues( vals,rowNo );		// 6.2.1.0 (2015/03/13)
			}
		};

		// 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
		helper.setSkipRowCount( getSkipRowCount() );		// 6.1.0.0 (2014/12/26) 外部からスキップ行数を指定。

		helper.setNullBreakClm( nullBreakClm );				// 6.2.0.0 (2015/02/27) 外部からnullBreakClmを指定。
		helper.setNullSkipClm( nullSkipClm );				// 6.2.3.0 (2015/05/01) 外部からnullSkipClmを指定。
		helper.setDebug( isDebug );							// 6.2.0.0 (2015/02/27)

		// 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
		helper.setConstData( constKeys , constAdrs );		// 6.1.0.0 (2014/12/26) 外部から固定値情報を指定。
		helper.setNames( columns , isUseNumber() );			// 6.1.0.0 (2014/12/26) 外部からカラム名配列を指定。

		// 6.2.4.2 (2015/05/29) POIUtil を使わず、EventReader_XLS、EventReader_XLSX を直接呼び出します。
		final String SUFIX = FileInfo.getSUFIX( file );
		if( "xls".equalsIgnoreCase( SUFIX ) ) {
			new EventReader_XLS().eventReader( file,helper );
		}
		else if( "xlsx".equalsIgnoreCase( SUFIX ) || "xlsm".equalsIgnoreCase( SUFIX ) ) {
			new EventReader_XLSX().eventReader( file,helper );
		}
		else {
			final String errMsg = "拡張子は、xls,xlsx,xlsm にしてください。[" + file + "]" ;
			throw new OgRuntimeException( errMsg );
		}

		// 最後まで、#NAME が見つから無かった場合
		if( !helper.isNameSet() ) {
			final String errMsg = "最後まで、#NAME が見つかりませんでした。"
							+ CR
							+ "ファイル形式が異なるか、もしくは損傷している可能性があります。"
							+ "Class=[Excel], File=[" + file + "]"
							+ CR ;
			throw new HybsSystemException( errMsg );
		}

		if( isDebug ) { System.out.println( "  TableReader End." ) ; }
	}
}

==> openGion8.5.2.0_src.zip: uap/webapps/gf/src/org/opengion/plugin/io/TableReader_Excel.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.io;

import org.opengion.fukurou.system.OgRuntimeException ;		// 6.4.2.0 (2016/01/29)
import java.io.File;										// 6.2.0.0 (2015/02/27)
import java.util.Arrays;									// 6.8.2.4 (2017/11/20)

import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.FileInfo;					// 6.2.4.2 (2015/05/29)
import org.opengion.fukurou.model.TableModelHelper;			// 6.2.0.0 (2015/02/27) クラス名変更
import org.opengion.fukurou.model.EventReader_XLS;			// 6.2.4.2 (2015/05/29)
import org.opengion.fukurou.model.EventReader_XLSX;			// 6.2.4.2 (2015/05/29)
// import org.opengion.fukurou.model.FileOperation;			// 8.0.0.1 (2021/10/08)
// import org.opengion.fukurou.util.FileUtil;					// 8.0.0.1 (2021/10/08)

import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.io.AbstractTableReader;		// 6.2.0.0 (2015/02/27)
// import org.opengion.hayabusa.io.HybsFileOperationFactory;	// 8.0.0.1 (2021/10/08)

import static org.opengion.fukurou.system.HybsConst.CR ;	// 6.2.2.0 (2015/03/27)

/**
 * POI による、EXCELバイナリファイルを読み取る実装クラスです。
 *
 * ファイル名、シート名を指定して、データを読み取ることが可能です。
 * 第一カラムが # で始まる行は、コメント行なので、読み飛ばします。
 * カラム名の指定行で、カラム名が null の場合は、その列は読み飛ばします。
 *
 * 入力形式は、openXML形式にも対応しています。
 * ファイルの内容に応じて、.xlsと.xlsxのどちらで読み取るかは、内部的に
 * 自動判定されます。
 *
 * @og.rev 3.5.4.8 (2004/02/23) 新規作成
 * @og.rev 4.3.6.7 (2009/05/22) ooxml形式対応
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class TableReader_Excel extends AbstractTableReader {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "8.0.0.2 (2021/10/15)" ;

	/**
	 * デフォルトコンストラクター
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public TableReader_Excel() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * DBTableModel から 各形式のデータを作成して、BufferedReader より読み取ります。
	 * コメント/空行を除き、最初の行は、必ず項目名が必要です。
	 * それ以降は、コメント/空行を除き、データとして読み込んでいきます。
	 * このメソッドは、EXCEL 読み込み時に使用します。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @og.rev 5.1.6.0 (2010/05/01) columns 処理 追加
	 * @og.rev 5.1.6.0 (2010/05/01) skipRowCountの追加
	 * @og.rev 5.1.8.0 (2010/07/01) Exception をきちっと記述(InvalidFormatException)
	 * @og.rev 5.2.1.0 (2010/10/01) setTableColumnValues メソッドを経由して、テーブルにデータをセットする。
	 * @og.rev 5.5.1.2 (2012/04/06) HeaderData を try の上にだし、エラーメッセージを取得できるようにする。
	 * @og.rev 5.5.7.2 (2012/10/09) sheetNos 追加による複数シートのマージ読み取りサポート
	 * @og.rev 5.5.8.2 (2012/11/09) HeaderData に デバッグフラグを渡します。
	 * @og.rev 6.0.2.5 (2014/10/31) debug=true 時に、進捗が見えるようにします。
	 * @og.rev 6.1.0.0 (2014/12/26) Excel関係改善
	 * @og.rev 6.2.0.0 (2015/02/27) TableReader クラスの呼び出し元メソッドの共通化(EXCEL,TEXT)。新規
	 * @og.rev 6.2.1.0 (2015/03/13) TableReaderModel を外部からセットします。
	 * @og.rev 6.2.3.0 (2015/05/01) 行読み飛ばし nullSkipClm追加
	 * @og.rev 6.2.4.2 (2015/05/29) POIUtil を使わず、EventReader_XLS、EventReader_XLSX を直接呼び出します。
	 * @og.rev 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
	 * @og.rev 8.0.0.1 (2021/10/08) OPCPackage が、ｸﾗｳﾄﾞﾌｧｲﾙを処理できない…みたい。
	 * @og.rev 8.0.0.2 (2021/10/15) TableReader系は、ｸﾗｳﾄﾞから、ﾛｰｶﾙﾌｧｲﾙに移してから処理する。
	 *
	 * @param   file 読み取り元ファイル名(ﾛｰｶﾙﾌｧｲﾙ)
	 * @param   enc ファイルのエンコード文字列(未使用)
	 */
	@Override
	public void readDBTable( final File file , final String enc ) {
		final boolean isDebug = isDebug();			// 5.5.7.2 (2012/10/09) デバッグ情報

	//	if( isDebug ) { System.out.println( " Filename=" + file ) ; }

//		// 8.0.0.1 (2021/10/08) OPCPackage が、ｸﾗｳﾄﾞﾌｧｲﾙを処理できない…みたい。
//		// ｸﾗｳﾄﾞのﾌｧｲﾙをﾛｰｶﾙにｺﾋﾟｰしています。
//		final File file = new File(inFile.toString());		// 後で修正
//		if( inFile instanceof FileOperation ) {
//			FileUtil.copy( inFile,file );
//		}

		final TableModelHelper helper = new TableModelHelper() {
			private boolean[] useShtNo ;			// 6.1.0.0 (2014/12/26) 読み取り対象のシート管理

			/**
			 * シートの数のイベントが発生します。
			 *
			 * 処理の開始前に、シートの数のイベントが発生します。
			 * これを元に、処理するシート番号の選別が可能です。
			 * 初期実装は、されていません。
			 *
			 * @og.rev 6.1.0.0 (2014/12/26) Excel関係改善
			 * @og.rev 6.2.6.0 (2015/06/19) #csv2ArrayExt(String,int)の戻り値を、文字列配列から数字配列に変更。
			 * @og.rev 5.9.9.2 (2016/06/17) シート名指定が効果ないため修正
			 *
			 * @param   size  シートの数
			 */
			@Override
			public void sheetSize( final int size ) {
				if( isDebug ) { System.out.println( " sheetSize=" + size ) ; }
				// 5.5.7.2 (2012/10/09) 複数シートのマージ読み取り。 sheetNos の指定が優先される。
				useShtNo = new boolean[size];			// シート数だけ、配列を作成する。
				if( sheetNos != null && sheetNos.length() > 0 ) {
					Integer[] sheetList = StringUtil.csv2ArrayExt( sheetNos , size-1 );	// 最大シート番号は、シート数-1
					for( int i=0; i<sheetList.length; i++ ) {
						useShtNo[sheetList[i]] = true;	// 読み取り対象のシート番号のみ、ture にセット
					}
				}
				else if ( sheetName == null || sheetName.length() == 0 ) { //  5.9.9.2 (2016/06/17) sheetNameを見ないと必ず先頭が読み込まれる
					useShtNo[0] = true;					// 一番目のシート
				}
			}

			/**
			 * シートの読み取り開始時にイベントが発生します。
			 *
			 * 新しいシートの読み取り開始毎に、１回呼ばれます。
			 * 戻り値が、true の場合は、そのシートの読み取りを継続します。
			 * false の場合は、そのシートの読み取りは行わず、次のシートまで
			 * イベントは発行されません。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 7.3.1.1 (2021/02/25) 現在実行中のシート名をセットする
			 *
			 * @param   shtNm  シート名
			 * @param   shtNo  シート番号(0～)
			 * @return  true:シートの読み取り処理を継続します/false:このシートは読み取りません。
			 */
			@Override
			public boolean startSheet( final String shtNm,final int shtNo ) {
	//			if( isDebug ) { System.out.println( " Sheet[" + shtNo + "]=" + shtNm ) ; }
				super.startSheet( shtNm , shtNo );		// cnstData の呼び出しの為。無しで動くようにしなければ…

//				return  useShtNo  != null && useShtNo[shtNo] ||
//						sheetName != null && sheetName.equalsIgnoreCase( shtNm ) ;				// 6.9.7.0 (2018/05/14) PMD Useless parentheses.

				final boolean isUseSheet = useShtNo  != null && useShtNo[shtNo] ||
									sheetName != null && sheetName.equalsIgnoreCase( shtNm ) ;	// 6.9.7.0 (2018/05/14) PMD Useless parentheses.

				if( isUseSheet ) { shtName( shtNm ); }	// 7.3.1.1 (2021/02/25)

				return isUseSheet;
			}

			/**
			 * カラム名配列がそろった段階で、イベントが発生します。
			 *
			 * openGion での標準的な処理は、colNo==0 の時に、val の先頭が、#NAME
			 * で始まるレコードを、名前配列として認識します。
			 * #value( String,int,int ) で、この #NAME だけは、継続処理されます。
			 * その上で、#NAME レコードが終了した時点で、カラム名配列が完成するので
			 * そこで初めて、このメソッドが呼ばれます。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 6.1.0.0 (2014/12/26) omitNames 属性を追加
			 * @og.rev 6.8.2.4 (2017/11/20) isDebug 処理を追加
			 *
			 * @param   names  カラム名配列(可変長引数)
			 * @see		#value( String,int,int )
			 */
			@Override
			public void columnNames( final String[] names ) {
				if( isDebug ) { System.out.println( " names=" + Arrays.toString( names ) ) ; }
				setTableDBColumn( names ) ;
			}

			/**
			 * #NAME のオリジナルカラム名配列がそろった段階で、イベントが発生します。
			 *
			 * @og.rev 7.3.1.3 (2021/03/09) #NAMEのオリジナルを取得できるようにします。
			 *
			 * @param   names  カラム名配列
			 */
			@Override
			public void originalNames( final String[] names ) {
				setOriginalNames( names ) ;
			}

			/**
			 * row にあるセルのオブジェクト値がそろった段階で、イベントが発生します。
			 *
			 * @og.rev 6.0.3.0 (2014/11/13) 新規作成
			 * @og.rev 6.2.1.0 (2015/03/13) setTableColumnValuesに、行番号を引数に追加
			 * @og.rev 6.8.2.4 (2017/11/20) isDebug 処理を追加
			 *
			 * @param   vals    文字列値の１行分の配列
			 * @param   rowNo   行番号(0～)
			 */
			@Override
			public void values( final String[] vals,final int rowNo ) {
				if( isDebug && rowNo % 100 == 0 ) { System.out.println( "   rowNo=" + rowNo ) ; }
				setTableColumnValues( vals,rowNo );		// 6.2.1.0 (2015/03/13)
			}
		};

		// 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
		helper.setSkipRowCount( getSkipRowCount() );		// 6.1.0.0 (2014/12/26) 外部からスキップ行数を指定。

		helper.setNullBreakClm( nullBreakClm );				// 6.2.0.0 (2015/02/27) 外部からnullBreakClmを指定。
		helper.setNullSkipClm( nullSkipClm );				// 6.2.3.0 (2015/05/01) 外部からnullSkipClmを指定。
		helper.setDebug( isDebug );							// 6.2.0.0 (2015/02/27)

		// 6.4.1.2 (2016/01/22) setConstData と、setNames は、内部処理が走るため、他の設定が終わってから呼び出す。
		helper.setConstData( constKeys , constAdrs );		// 6.1.0.0 (2014/12/26) 外部から固定値情報を指定。
		helper.setNames( columns , isUseNumber() );			// 6.1.0.0 (2014/12/26) 外部からカラム名配列を指定。

		// 6.2.4.2 (2015/05/29) POIUtil を使わず、EventReader_XLS、EventReader_XLSX を直接呼び出します。
		final String SUFIX = FileInfo.getSUFIX( file );
		if( "xls".equalsIgnoreCase( SUFIX ) ) {
			new EventReader_XLS().eventReader( file,helper );
		}
		else if( "xlsx".equalsIgnoreCase( SUFIX ) || "xlsm".equalsIgnoreCase( SUFIX ) ) {
			new EventReader_XLSX().eventReader( file,helper );
		}
		else {
			final String errMsg = "拡張子は、xls,xlsx,xlsm にしてください。[" + file + "]" ;
			throw new OgRuntimeException( errMsg );
		}

		// 最後まで、#NAME が見つから無かった場合
		if( !helper.isNameSet() ) {
			final String errMsg = "最後まで、#NAME が見つかりませんでした。"
							+ CR
							+ "ファイル形式が異なるか、もしくは損傷している可能性があります。"
							+ "Class=[Excel], File=[" + file + "]"
							+ CR ;
			throw new HybsSystemException( errMsg );
		}

		if( isDebug ) { System.out.println( "  TableReader End." ) ; }
	}
}
